@@ -5,7 +5,12 @@ use warnings;
 package BSON::ObjectId;
 # ABSTRACT: ObjectId data element for BSON
 
-our $VERSION = '0.12';
+our $VERSION = '0.16';
+
+# if threads are in use, we need threads::shared loaded, too
+if ( $INC{"threads.pm"} ) {
+    require threads::shared;
+}
 
 use Carp;
 use Sys::Hostname;
@@ -16,12 +21,27 @@ use overload
   '==' => \&op_eq,
   'eq' => \&op_eq;
 
-my $_inc : shared = 0;
+my $_inc : shared;
+{
+    lock($_inc);
+    $_inc = int(rand(0xFFFFFF));
+}
+
+my $_host = substr( md5(hostname), 0, 3 );
 
 sub new {
     my ( $class, $value ) = @_;
     my $self = bless {}, $class;
-    $self->value( $value || _generate() );
+    if ( $value ) {
+        $self->value( $value || _generate() );
+    }
+    else {
+        $self->{value} =
+            pack( 'N', time )
+          . $_host
+          . pack( 'n', $$ % 0xFFFF )
+          . substr( pack( 'N', do { lock($_inc); $_inc++; $_inc %= 0xFFFFFF }), 1, 3);
+    }
     return $self;
 }
 
@@ -55,15 +75,6 @@ sub op_eq {
     return ref($self) eq ref($other) && $self->value eq $other->value;
 }
 
-sub _generate {
-    my $self = shift;
-    my $time = pack( 'N', shift || time );
-    my $host = substr( md5(hostname), 0, 3 );
-    my $proc = pack( 'n', $$ % 0xFFFF );
-    my $inc  = substr( pack( 'N', $_inc++ % 0xFFFFFF ), 1, 3 );
-    return $time . $host . $proc . $inc;
-}
-
 sub _from_s {
     my @a = split( //, shift );
     my $oid = '';
@@ -87,7 +98,7 @@ BSON::ObjectId - ObjectId data element for BSON
 
 =head1 VERSION
 
-version 0.12
+version 0.16
 
 =head1 SYNOPSIS
 
